@@ -22,6 +22,7 @@ from typing import Optional, Union
 import requests
 from logzero import setup_logger
 from tabulate import tabulate
+from loguru import logger as ulogger
 
 from ._device import Device
 from ._imagemounter import cache_developer_image
@@ -85,21 +86,23 @@ def cmd_list(args: argparse.Namespace):
     ds = um.device_list()
     if args.usb:
         ds = [info for info in ds if info.conn_type == ConnectionType.USB]
-    
+
     if args.one:
         for info in ds:
             print(info.udid)
         return
 
-    headers = ['UDID', 'NAME', 'MarketName', 'ProductVersion', "ConnType"]
-    keys = ["udid", "name", "market_name", "product_version", "conn_type"]
+    headers = ['UDID', 'SerialNumber', 'NAME', 'MarketName', 'ProductVersion', "ConnType"]
+    keys = ["udid", "serial", "name", "market_name", "product_version", "conn_type"]
     tabdata = []
     for dinfo in ds:
         udid, conn_type = dinfo.udid, dinfo.conn_type
         try:
             _d = Device(udid, um)
             name = _d.name
-            tabdata.append([udid, name, MODELS.get(_d.product_type, "-"), _d.product_version, conn_type])
+            serial = _d.get_value("SerialNumber")
+            tabdata.append([udid, serial, name, MODELS.get(_d.product_type, "-"), _d.product_version, conn_type])
+
         except MuxError:
             name = ""
     if _json:
@@ -161,8 +164,9 @@ def cmd_install(args: argparse.Namespace):
     bundle_id = d.app_install(args.filepath_or_url)
 
     if args.launch:
-        pid = d.instruments.app_launch(bundle_id)
-        logger.info("Launch %r, process pid: %d", bundle_id, pid)
+        with d.connect_instruments() as ts:
+            pid = ts.app_launch(bundle_id)
+            logger.info("Launch %r, process pid: %d", bundle_id, pid)
 
 
 def cmd_uninstall(args: argparse.Namespace):
@@ -247,6 +251,7 @@ def cmd_xctest(args: argparse.Namespace):
     Run XCTest required WDA installed.
     """
     if args.debug:
+        ulogger.enable(PROGRAM_NAME)
         setup_logger(LOG.xctest, level=logging.DEBUG)
 
     d = _udid2device(args.udid)
@@ -307,14 +312,39 @@ def cmd_applist(args: argparse.Namespace):
         #     (bundle_path, bundle_id, info['DisplayName'],
         #         info.get('Version', ''), info['Type'])))
 
+def cmd_energy(args: argparse.Namespace):
+    d = _udid2device(args.udid)
+    ts = d.connect_instruments()
+    try:
+        pid = ts.app_launch(args.bundle_id,
+                                       args=args.arguments,
+                                       kill_running=args.kill)
+        ts.start_energy_sampling(pid)
+        while True:
+            ret = ts.get_process_energy_stats(pid)
+            if ret != None:
+                print(json.dumps(ret))
+            time.sleep(1.0)
+    except ServiceError as e:
+        sys.exit(e)
 
 def cmd_launch(args: argparse.Namespace):
     d = _udid2device(args.udid)
+
+    env = {}
+    for kv in args.env or []:
+        key, val = kv.split(":", 1)
+        env[key] = val
+    if env:
+        logger.info("App launch env: %s", env)
+
     try:
-        pid = d.instruments.app_launch(args.bundle_id,
-                                       args=args.arguments,
-                                       kill_running=args.kill)
-        print("PID:", pid)
+        with d.connect_instruments() as ts:
+            pid = ts.app_launch(args.bundle_id,
+                                        app_env=env,
+                                        args=args.arguments,
+                                        kill_running=args.kill)
+            print("PID:", pid)
     except ServiceError as e:
         sys.exit(e)
 
@@ -334,8 +364,9 @@ def cmd_kill(args: argparse.Namespace):
 
 def cmd_system_info(args):
     d = _udid2device(args.udid)
-    sinfo = d.instruments.system_info()
-    pprint(sinfo)
+    with d.connect_instruments() as ts:
+        sinfo = ts.system_info()
+        pprint(sinfo)
 
 
 def cmd_battery(args: argparse.Namespace):
@@ -344,7 +375,42 @@ def cmd_battery(args: argparse.Namespace):
     if args.json:
         _print_json(power_info)
     else:
-        pprint(power_info)
+        if power_info['Status'] != "Success":
+            pprint(power_info)
+            return
+        # dump power info
+        info = power_info['Diagnostics']['IORegistry']
+        indexes = (
+            ('CurrentCapacity', '当前电量', '%'),
+            ('CycleCount', '充电次数', '次'),
+            ('AbsoluteCapacity', '当前电量', 'mAh'),
+            ('NominalChargeCapacity', '实际容量', 'mAh'),
+            ('DesignCapacity', '设计容量', 'mAh'),
+            ('NominalChargeCapacity', '电池寿命', lambda cap: '{}%'.format(round(cap / info['DesignCapacity'] * 100))),
+            ('Serial', '序列号', ''),
+            ('Temperature', '电池温度', '/100℃'),
+            ('Voltage', '当前电压', 'mV'),
+            ('BootVoltage', '开机电压', 'mV'),
+            (('AdapterDetails', 'Watts'), '充电器功率', 'W'),
+            (('AdapterDetails', 'Voltage'), '充电器电压', 'mV'),
+            ('InstantAmperage', '当前电流', 'mA'),
+            ('UpdateTime', '更新时间', lambda v: datetime.fromtimestamp(v).strftime("%Y-%m-%d %H:%M:%S")),
+        )
+        # 数据一般20s更新一次
+
+        for keypath, cn_name, unit in indexes:
+            value = info
+            if isinstance(keypath, str):
+                value = info[keypath]
+            else:
+                value = info
+                for key in keypath:
+                    value = value[key]
+            if callable(unit):
+                value = unit(value)
+                unit = ""
+            print("{:10s}{}{}".format(cn_name, value, unit))
+
 
 def cmd_crashreport(args: argparse.Namespace):
     d = _udid2device(args.udid)
@@ -373,6 +439,9 @@ def cmd_developer(args: argparse.Namespace):
                 except requests.HTTPError:
                     break
         #     print("finish cache developer image {}".format(version))
+    elif args.list:
+        d = _udid2device(args.udid)
+        print(d.imagemounter.lookup())
     else:
         d = _udid2device(args.udid)
         d.mount_developer_image()
@@ -420,7 +489,7 @@ def cmd_syslog(args: argparse.Namespace):
     s = d.start_service("com.apple.syslog_relay")
     try:
         while True:
-            text = s.recv().decode('utf-8')
+            text = s.psock.recv().decode('utf-8')
             print(text, end='', flush=True)
     except (BrokenPipeError, IOError):
         # Python flushes standard streams on exit; redirect remaining output
@@ -431,7 +500,7 @@ def cmd_syslog(args: argparse.Namespace):
 
 def cmd_dump_fps(args):
     d = _udid2device(args.udid)
-    for data in d.instruments.iter_opengl_data():
+    for data in d.connect_instruments().iter_opengl_data():
         if isinstance(data, str):
             continue
         fps = data['CoreAnimationFramesPerSecond']
@@ -509,7 +578,8 @@ def cmd_fsync(args: argparse.Namespace):
 def cmd_ps(args: argparse.Namespace):
     d = _udid2device(args.udid)
     app_infos = list(d.installation.iter_installed(app_type=None))
-    ps = list(d.instruments.app_process_list(app_infos))
+    with d.connect_instruments() as ts:
+        ps = list(ts.app_process_list(app_infos))
 
     lens = defaultdict(int)
     json_data = []
@@ -576,7 +646,7 @@ def cmd_savesslfile(args: argparse.Namespace):
     pathlib.Path(f"ssl/{d.udid}_all.pem").write_bytes(
         pr['HostPrivateKey']
         + pr['HostCertificate']
-        + pr['RootCertificate']
+        # + pr['RootCertificate']
     )
 
 
@@ -592,9 +662,7 @@ _commands = [
              dict(args=['--json'],
                   action='store_true',
                   help='output in json format'),
-             dict(args=['--usb'],
-                  action='store_true',
-                  help='usb USB device'),
+             dict(args=['--usb'], action='store_true', help='usb USB device'),
              dict(args=['-1'],
                   dest="one",
                   action='store_true',
@@ -643,7 +711,10 @@ _commands = [
     dict(action=cmd_applist,
          command="applist",
          flags=[
-             dict(args=['--type'], default='user', help='filter app with type', choices=['user', 'system', 'all'])
+             dict(args=['--type'],
+                  default='user',
+                  help='filter app with type',
+                  choices=['user', 'system', 'all'])
          ],
          help="list packages"),
     dict(action=cmd_battery,
@@ -686,6 +757,19 @@ _commands = [
          help='wait for device attached'),
     dict(action=cmd_launch,
          command="launch",
+         flags=[
+             dict(args=['--kill'],
+                  action='store_true',
+                  help='kill app if running'),
+             dict(args=["bundle_id"], help="app bundleId"),
+             dict(args=['arguments'], nargs='*', help='app arguments'),
+             dict(args=['-e', '--env'],
+                  action='append',
+                  help="set env with format key:value, support multi -e"),
+         ],
+         help="launch app with bundle_id"),
+    dict(action=cmd_energy,
+         command="energy",
          flags=[
              dict(args=['--kill'],
                   action='store_true',
@@ -779,16 +863,27 @@ _commands = [
     dict(action=cmd_crashreport,
          command="crashreport",
          flags=[
-             dict(args=['--list'], action='store_true', help='list all crash files'),
-             dict(args=['--keep'], action='store_true', help="copy but do not remove crash reports from device"),
-             dict(args=['--clear'], action='store_true', help='clear crash files'),
-             dict(args=['output_directory'], nargs="?", help='The output dir to save crash logs synced from device'),
+             dict(args=['-l', '--list'],
+                  action='store_true',
+                  help='list all crash files'),
+             dict(args=['-k', '--keep'],
+                  action='store_true',
+                  help="copy but do not remove crash reports from device"),
+             dict(args=['-c', '--clear'],
+                  action='store_true',
+                  help='clear crash files'),
+             dict(args=['output_directory'],
+                  nargs="?",
+                  help='The output dir to save crash logs synced from device'),
          ],
          help="crash log tools"),
     dict(action=cmd_dump_fps, command='dumpfps', help='dump fps'),
     dict(action=cmd_developer,
          command="developer",
          flags=[
+             dict(args=["-l", "--list"],
+                  action="store_true",
+                  help="list mount information"),
              dict(args=['--download-all'],
                   action="store_true",
                   help="download all developer to local")
@@ -829,6 +924,7 @@ def main():
     parser.add_argument("-v", "--version", action="store_true", help="show current version"),
     parser.add_argument("-u", "--udid", help="specify unique device identifier")
     parser.add_argument("--socket", help="usbmuxd listen address, host:port or local-path")
+    parser.add_argument("--trace", action="store_true", help="show trace info")
 
     subparser = parser.add_subparsers(dest='subparser')
     actions = {}
@@ -857,6 +953,9 @@ def main():
         # show_upgrade_message()
         return
 
+    if args.trace:
+        ulogger.enable(PROGRAM_NAME)
+        
     # log setup
     setup_logger(LOG.main,
         level=logging.DEBUG if os.getenv("DEBUG") in ("1", "on", "true") else logging.INFO)
