@@ -693,11 +693,13 @@ class BaseDevice():
     def app_start(self,
                   bundle_id: str,
                   args: Optional[list] = [],
-                  kill_running: bool = False) -> int:
+                  kill_running: bool = True) -> int:
         """
         start application
         
         return pid
+
+        Note: kill_running better to True, if set to False, launch 60 times will trigger instruments service stop
         """
         with self.connect_instruments() as ts:
             return ts.app_launch(bundle_id, args=args, kill_running=kill_running)
@@ -788,7 +790,8 @@ class BaseDevice():
         return DTXService(conn)
 
     # 2022-08-24 add retry delay, looks like sometime can recover
-    @retry((ssl.SSLError, socket.timeout), delay=10, jitter=1, tries=3, logger=logging)
+    # BrokenPipeError(ConnectionError)
+    @retry((ssl.SSLError, socket.timeout, BrokenPipeError), delay=10, jitter=1, tries=3, logger=logging)
     def connect_instruments(self) -> ServiceInstruments:
         """ start service for instruments """
         if self.major_version() >= 14:
